from django.conf.urls import url, include
from django.conf.urls.static import static
from django.conf import settings
from django.contrib import admin
from django.contrib.auth import views as auth_views

from rest_framework import permissions
from drf_yasg.views import get_schema_view
from drf_yasg import openapi

schema_view = get_schema_view(
    openapi.Info(
        title="Snippets API",
        default_version='v1',
        description="Test description",
        terms_of_service="https://www.google.com/policies/terms/",
        contact=openapi.Contact(email="contact@snippets.local"),
        license=openapi.License(name="BSD License"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)

urlpatterns = [
    url(r'^api/v1/swagger(?P<format>\.json|\.yaml)$',
        schema_view.without_ui(cache_timeout=0), name='schema-json'),
    url(r'^api/v1/swagger/$', schema_view.with_ui('swagger',
                                                  cache_timeout=0), name='schema-swagger-ui'),
    url(r'^api/v1/redoc/$', schema_view.with_ui('redoc',
                                                cache_timeout=0), name='schema-redoc'),
    url(r'^admin/', admin.site.urls),
    url(r'^api/v1/', include('social_django.urls', namespace='social')),
    url(r'^api/v1/homework/', include('homework.urls')),
    url(r'^api/v1/', include('stats.urls')),
    url(r'^api/v1/', include('account.urls')),
    url(r'^api/v1/', include('news.urls')),
    url(r'^api/v1/', include('document.urls')),
    url(r'^api/v1/', include('groups.urls')),
    url(r'^api/v1/', include('mentors.urls')),
    url(r'^api/v1/', include('images.urls')),
    url(r'^api/v1/logout/$', auth_views.LogoutView.as_view(), name='logout'),
]

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL,
                          document_root=settings.MEDIA_ROOT)
